; ***********************************************************************************
; * Name		:	G. Mayer														*
; *	Description	:	Subroutines zum Gong											*
; * Datum		:	01.06.2014														*
; *																					*
; *  2014 dazkeiler.de																*
; *																					*
; *	Folgende Subroutines sind vorhanden												*
; * ===================================												*
; *																					*
; * PlaySound																		*
; * InitRAMAndReg 																	*
; *																					*
; ***********************************************************************************

; ***********************************************************************************
; * PlaySound 																		*
; ***********************************************************************************
PlaySound:
;	sbi		StatusLEDPort,StatusLED
	in		XL,SWPort
	andi	XL,0b00000011
	cpi		XL,0
	brne	PlaySound_Not1

    ldi 	ZH,High(TAB_Sound01<<1)             			; Initialize Z pointer
    ldi 	ZL,Low(TAB_Sound01<<1)
	rjmp	PlaySound_NextTon

PlaySound_Not1:
	cpi		XL,1
	brne	PlaySound_Not2

    ldi 	ZH,High(TAB_Sound02<<1)             			; Initialize Z pointer
    ldi 	ZL,Low(TAB_Sound02<<1)
	rjmp	PlaySound_NextTon

PlaySound_Not2:
	cpi		XL,2
	brne	PlaySound_Not3

    ldi 	ZH,High(TAB_Sound03<<1)             			; Initialize Z pointer
    ldi 	ZL,Low(TAB_Sound03<<1)
	rjmp	PlaySound_NextTon

PlaySound_Not3:
	cpi		XL,3
	brne	PlaySound_Not4

    ldi 	ZH,High(TAB_Sound04<<1)             			; Initialize Z pointer
    ldi 	ZL,Low(TAB_Sound04<<1)
	rjmp	PlaySound_NextTon

PlaySound_Not4:
	rjmp	PlaySound_End

PlaySound_NextTon:
    lpm                     							; Note(0-11 + oktavNr 0 oder 1) aus Tabelle laden
	mov 	R_TP2,R_TABVAL       						; Note zwischenspeichern
	mov 	R_TP3,R_TABVAL
	inc 	ZL
	brne 	PlaySound_NextTon2
	inc 	ZH

PlaySound_NextTon2:										; Note in Frequenzen umwandeln (mit Tabelle TONFRQ)
    mov 	R_ZRegL,ZL         							; Z-Pointer zwischenspeichern
	mov 	R_ZRegH,ZH
	andi 	R_TP2,0x0f           						; Ton ohne Oktaven-Nummer
    lsl  	R_TP2                						; mal 2, weil 2 Oktaven in der Tabelle sind
    swap 	R_TP3
	andi 	R_TP3,0x0F
	add  	R_TP2,R_TP3            						; In tp2 steht jetzt der Index fr die tabelle tonfrq
    ldi 	ZH,High(TONFRQ<<1)              			; Initialize Z pointer
    ldi 	ZL,Low(TONFRQ<<1)
	add 	ZL,R_TP2                          			; Index addieren
	brne 	PlaySound_NextTon3
	inc 	ZH

PlaySound_NextTon3:
    lpm                       							; die echte Tonhhe laden
	mov 	R_TON,R_TABVAL
	mov 	ZL,R_ZRegL
	mov 	ZH,R_ZRegH              					; Z-Pointer wieder zurckspeichern
    lpm                     							; Tondauer (mit Enable-Bit) aus Tabelle laden
	mov 	R_TWait,R_TABVAL
	inc 	ZL
	brne 	PlaySound_NextTon4
	inc 	ZH

PlaySound_NextTon4:
	cpi 	R_TWait,0
    breq 	PlaySound_End     							; Sound zuende

    ldi 	R_SoundEN,1       							; Ton anschalten - als default
    sbrc 	R_TWait,0         							; berspringen, wenn bit 0 = 0
	ldi 	R_SoundEN,0       							; Ton aus wenn bit 0 = 1
	;andi 	TmWait,0xFE     	 						; Bit 0 lschen
	lsr 	R_TWait            							; Bit 0 vergessen
    mov 	R_TP3,R_TWait

PlaySound_Loop0:													; Schleife fr Tondauer
	wdr
	ldi 	R_TP2,BaseTime
	add 	R_TP2,R_SUM2

PlaySound_Loop1:
	wdr
    cp 		R_TP2,R_SUM2
	brne 	PlaySound_Loop1
	dec 	R_TP3
	brne 	PlaySound_Loop0 

	ldi 	R_SoundEN,0									; Kurze Unterbrechung (1/32 Note) vor dem nchsten Ton
	ldi 	R_TP2,BaseTime
	add 	R_TP2,R_SUM2

PlaySound_Loop2:
	wdr
    cp 		R_TP2,R_SUM2
	brne 	PlaySound_Loop2

    rjmp 	PlaySound_NextTon							; Nchster Ton aus Tabelle

PlaySound_End:
    ldi 	R_SoundEN,0
	cbi		SpeakerPort,Speaker
;	cbi		StatusLEDPort,StatusLED
	ret

; ***********************************************************************************
; * RAM und Register Initialisieren													*
; ***********************************************************************************
InitRAMAndReg:
	; SRAM lschen
	ldi		XL,Low(SRAM_START)
	ldi		XH,High(SRAM_START)
	clr		YL

InitRAMAndReg_SRAMLoop:
	st		X+,YL
	ldi		YH,High(RAM_MyEnd)
	cpi		XL,Low(RAM_MyEnd)
	cpc		XH,YH
	brne	InitRAMAndReg_SRAMLoop

	; R0 - R31 lschen
	ldi		XL,0
	ldi		XH,0
	ldi		YL,0

InitRAMAndReg_RegisterLoop:
	st		X+,YL
	cpi		XL,26
	brne	InitRAMAndReg_RegisterLoop

	clr		XL
	clr		XH
	clr		YL
	clr		YH
	clr		ZL
	clr		ZH
	ret
